#include <stdio.h>

#include "hardware.h"
#include "leds.h"
#include "uart.h"
#include "switch.h"

static void rfxx_port_init(void)
{
	HI_SEL();
	HI_SDI();
	LOW_SCK();
	SEL_OUTPUT();
	SDI_OUTPUT();
	SDO_INPUT();
	PORT_SDO |= (1<<RFXX_SDO);
	SCK_OUTPUT();
	IRQ_IN();
}

static unsigned int rfxx_write_cmd(unsigned int cmd)
{
	unsigned char i;
	unsigned int temp = 0;

	LOW_SCK();
	LOW_SEL();
	for (i=0;i<16;i++) {
		LOW_SCK();
		if (cmd & 0x8000)
			HI_SDI();
		else
			LOW_SDI();
		HI_SCK();

		temp <<= 1;
		if (SDO_HI())
			temp |= 0x0001;

		cmd <<= 1;
	}
	LOW_SCK();
	HI_SEL();
	return temp;
}

static void rf12_init(void)
{
	rfxx_write_cmd(0x80E7);
	rfxx_write_cmd(0x82d9);
	rfxx_write_cmd(0xa640);
	rfxx_write_cmd(0xc647);
	rfxx_write_cmd(0x94a0);
	rfxx_write_cmd(0xc2ac);
	rfxx_write_cmd(0xca81);
	rfxx_write_cmd(0xced4);
	rfxx_write_cmd(0xc483);
	rfxx_write_cmd(0x9850);
	rfxx_write_cmd(0xcc77);
	rfxx_write_cmd(0xe000);
	rfxx_write_cmd(0xc800);
	rfxx_write_cmd(0xc040);
}

/*
static void rf12_send(unsigned char byte)
{
	while (RF_IRQ_PIN & RF_IRQ_BIT);
	rfxx_write_cmd(0xb800 | byte);
}
*/

static unsigned char rf12_recv(void)
{
	unsigned int data;
	WAIT_IRQ_LOW();
	rfxx_write_cmd(0x0000);
	data = rfxx_write_cmd(0xb000);
	return data & 0x00ff;
}

int main()
{
	unsigned int i, j;
	unsigned char crc;
	unsigned char mesg_crc;
	unsigned char mesg[256];
	char buf[64];

	leds_init();
	switch_init();
	uart_init();

	LEDG_OFF();
	LEDR_OFF();
	LEDG_OUTPUT();
	LEDR_OUTPUT();

	for (i=0;i<3;i++) {
		_delay_ms(200);
		LEDG_ON();
		LEDR_ON();
		_delay_ms(200);
		LEDG_OFF();
		LEDR_OFF();
	}

	LEDG_OFF();
	LEDR_OFF();

	rfxx_port_init();
	rf12_init();

	rfxx_write_cmd(0xca81);

	sei();
	
	while (1) {
		rfxx_write_cmd(0xca83);
		crc = 0;

		for (i=0;i<16*16;i++) {
			mesg[i] = rf12_recv();
			crc += mesg[i];
		}
		mesg_crc = rf12_recv();

		rfxx_write_cmd(0xca81);

		j=0;
		for (i=0;i<16;i++)	{
			sprintf(buf, "%02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X %02X \r\n", mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++], mesg[j++]);
			uart_send_string(buf);
		}
		sprintf(buf, "crc=%02X (mesg_crc=%02X)\r\n", crc, mesg_crc);
		uart_send_string(buf);

		if (crc==mesg_crc) {
			LEDG_ON();
			_delay_ms(200);
			LEDG_OFF();
		}
	}
	return 0;
}
